﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Linq;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Workflow;
using Microsoft.SharePoint.WorkflowActions;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.Xsl;
using System.IO;
using System.Reflection;
using Winnovative.WnvHtmlConvert;
using System.Web.UI;

namespace ExpenseReportWorkflow.Workflow1
{
    public sealed partial class Workflow1 : SequentialWorkflowActivity
    {
        public Workflow1()
        {
            InitializeComponent();
        }

        public Guid workflowId = default(System.Guid);
        public SPWorkflowActivationProperties workflowProperties = new SPWorkflowActivationProperties();

        private void RetrieveFormData_ExecuteCode(object sender, EventArgs e)
        {
            SPFile file = onWorkflowActivated1.WorkflowProperties.Item.File;

            XmlTextReader reader = new XmlTextReader(file.OpenBinaryStream());

            XmlSerializer serializer = new XmlSerializer(
                typeof(SPWFiAExpenseReport));

            SPWFiAExpenseReport fields = (SPWFiAExpenseReport)serializer
                .Deserialize(reader);

            double total = 0;
            foreach (Expense expense in fields.Expenses)
            {
                total += expense.Amount;
            }

            file.Item["Total Again"] = total;
            file.Item.Update();
        }

        private void ArchiveAsPDF_ExecuteCode(object sender, EventArgs e)
        {
            string html = GetFormHTML();

            byte[] pdf = ConvertHTMLtoPDF(html);

            WritePDFtoArchive(pdf);
            WritePDFtoFileSystem(pdf);
        }

        private string GetFormHTML()
        {
            SPWeb web = onWorkflowActivated1.WorkflowProperties.Web;
            string libraryName =
                onWorkflowActivated1.WorkflowProperties.List.Title;
            string formName =
                onWorkflowActivated1.WorkflowProperties.Item.File.Title;

            SPFolder folder = web.Folders[libraryName];
            SPFile file = folder.Files[formName];

            XslCompiledTransform transform = new XslCompiledTransform();

            string resourceName = @"ExpenseReportWorkflow.view1.xsl";

            using (Stream res = Assembly.GetExecutingAssembly().
                GetManifestResourceStream(resourceName))
            {
                using (XmlTextReader stylesheet = new XmlTextReader(res))
                {
                    transform.Load(stylesheet);
                }
            }

            StringWriter sWriter = new StringWriter();
            HtmlTextWriter writer = new HtmlTextWriter(sWriter);

            using (XmlReader reader = new
                XmlTextReader(file.OpenBinaryStream()))
            {
                XmlTextWriter results = new
                    XmlTextWriter(writer.InnerWriter);

                transform.Transform(reader, results);
                reader.Close();
            }

            return sWriter.ToString();
        }

        private byte[] ConvertHTMLtoPDF(string html)
        {
            PdfConverter pdfConverter = new PdfConverter();

            pdfConverter.LicenseKey = null;

            pdfConverter.PdfDocumentOptions.PdfPageSize = PdfPageSize.A4;
            pdfConverter.PdfDocumentOptions.PdfCompressionLevel =
                PdfCompressionLevel.Normal;
            pdfConverter.PdfDocumentOptions.PdfPageOrientation =
                PDFPageOrientation.Portrait;
            pdfConverter.PdfDocumentOptions.ShowHeader = false;
            pdfConverter.PdfDocumentOptions.ShowFooter = false;

            pdfConverter.PdfDocumentOptions.GenerateSelectablePdf = true;

            byte[] pdfBytes = null;
            pdfBytes = pdfConverter.GetPdfBytesFromHtmlString(html);
            return pdfBytes;
        }

        private void WritePDFtoArchive(byte[] pdf)
        {
            SPFolder archive = onWorkflowActivated1.WorkflowProperties.
                Web.Folders["Archived Forms"];

            string filename = onWorkflowActivated1.WorkflowProperties.
                Item.File.Name.Replace(".xml", ".pdf");

            SPFile archivedFile = archive.Files.Add(archive.Url + "/" +
                filename, pdf, true);

            archive.Update();
        }

        private void WritePDFtoFileSystem(byte[] pdf)
        {
            string filename = onWorkflowActivated1.WorkflowProperties.Item.
                File.Name.Replace(".xml", ".pdf");

            FileStream s = new FileStream(
                "c:\\somefolder\\onserver\\harddrive\\" +
                filename, FileMode.Create);

            s.Write(pdf, 0, pdf.Length);
        }
    }
}
